from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from base64 import b64decode
from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto.Hash.HMAC import HMAC
from Crypto.Util import Counter

from nova.core import check_latest_version
from nova.core.exc import NovaError


class Decrypt:

    def __init__(self, stash_key, manager_provider, aws_profile=None, aws_region=None, aws_bucket=None):
        check_latest_version()

        self._aws_manager = manager_provider.aws_manager(aws_profile, aws_region or 'us-east-1')

        if aws_bucket is None:
            deployment_bucket_name = 'novastash_%s' % self._aws_manager.account_alias
        else:
            deployment_bucket_name = aws_bucket

        key = "%s.txt.enc" % stash_key
        existing_stash = self._aws_manager.s3_get(deployment_bucket_name, key)

        if existing_stash is None:
            raise NovaError("No stash '%s' found!" % stash_key)
        else:
            contents = existing_stash['Body'].read()
            metadata = existing_stash['Metadata']
            encryption_key = metadata['encryption-key']
            kms_response = self._aws_manager.kms_decrypt(b64decode(encryption_key), {})

            key = kms_response['Plaintext'][:32]
            hmac_key = kms_response['Plaintext'][32:]
            hmac = HMAC(hmac_key, msg=b64decode(contents), digestmod=SHA256)

            if hmac.hexdigest() != metadata['hmac']:
                raise NovaError("Computed HMAC on '%s' does not match stored HMAC" % stash_key)

            dec_ctr = Counter.new(128)
            decryptor = AES.new(key, AES.MODE_CTR, counter=dec_ctr)
            print(decryptor.decrypt(b64decode(contents)).decode("utf-8"))



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import sys
import git
import time
import boto3
import hashlib
import tempfile
from termcolor import colored
from boto3.s3.transfer import S3Transfer
from botocore.exceptions import ClientError, WaiterError

from nova.core import NovaError
from nova.core import query_yes_no
from nova.core.utils import tarfile_progress as tarfile
from nova.core.utils.s3_progress import ProgressPercentage
from nova.core.utils.cfn_waiter import CloudformationWaiter


class AwsManager(object):

    def __init__(self, profile, region, session=None):
        self.profile = profile
        self.region = region
        # if session:
        #     self._session = session
        # else:
        self._session = boto3.session.Session(profile_name=profile, region_name=region)
        self._account_alias = None
        self._s3_client = None
        self._cf_client = None
        self._cf_resource = None
        self._code_deploy_client = None
        self._iam_client = None
        self._kms_client = None
        self._route53_client = None

    @property
    def account_alias(self):
        if self._account_alias is None:
            self._account_alias = self.iam_client.list_account_aliases()['AccountAliases'][0]
        return self._account_alias

    @property
    def cloudformation(self):
        if self._cf_resource is None:
            self._cf_resource = self._session.resource('cloudformation')
        return self._cf_resource

    @property
    def cloudformation_client(self):
        if self._cf_client is None:
            self._cf_client = self._session.client('cloudformation')
        return self._cf_client

    @property
    def s3_client(self):
        if self._s3_client is None:
            self._s3_client = self._session.client('s3')
        return self._s3_client

    @property
    def code_deploy_client(self):
        if self._code_deploy_client is None:
            self._code_deploy_client = self._session.client('codedeploy')
        return self._code_deploy_client

    @property
    def iam_client(self):
        if self._iam_client is None:
            self._iam_client = self._session.client('iam')
        return self._iam_client

    @property
    def kms_client(self):
        if self._kms_client is None:
            self._kms_client = self._session.client('kms')
        return self._kms_client

    @property
    def route53_client(self):
        if self._route53_client is None:
            self._route53_client = self._session.client('route53')
        return self._route53_client

    def get_hosted_zone_names(self):
        return [z.get('Name') for z in self.route53_client.list_hosted_zones_by_name().get('HostedZones')]

    def create_bucket(self, deployment_bucket_name, msg):
        existing_bucket = None
        try:
            existing_bucket = self.s3_client.head_bucket(Bucket=deployment_bucket_name)
        except ClientError:
            print(colored(msg))
            self.s3_client.create_bucket(Bucket=deployment_bucket_name)
        return existing_bucket

    def s3_put(self, deployment_bucket_name, body, key, meta):
        self.s3_client.put_object(
            Bucket=deployment_bucket_name,
            Body=body,
            Key=key,
            Metadata=meta
        )

    def s3_get(self, deployment_bucket_name, key):
        existing_obj = None
        try:
            existing_obj = self.s3_client.get_object(Bucket=deployment_bucket_name, Key=key)
        except ClientError:
            pass
        return existing_obj

    def s3_head(self, deployment_bucket_name, key):
        existing_obj = None
        try:
            existing_obj = self.s3_client.head_object(Bucket=deployment_bucket_name, Key=key)
        except ClientError:
            pass
        return existing_obj

    def create_stack(self, service_name, cloudformation_template):
        stack_id = self.cloudformation.create_stack(
            StackName=service_name,
            TemplateBody=cloudformation_template,
            Capabilities=["CAPABILITY_IAM"]
        )

        print(colored(stack_id, color='green'))

        waiter = CloudformationWaiter(self.cloudformation)
        print(colored('Cloudformation stack creation in progress. Please check the AWS console!', color='green'))
        print(colored('Waiting on stack creation...', color='magenta'))
        waiter.waiter.wait(StackName=service_name)
        print(colored('Stack creation finished!', color='green'))
        return stack_id

    def get_stack(self, service_name):
        return self.cloudformation.Stack(service_name)

    def update_stack(self, service_name, cloudformation_template, changeset_id, cf_stack):
        try:
            cs_response = self.cloudformation_client.create_change_set(
                StackName=cf_stack.stack_name,
                TemplateBody=cloudformation_template,
                Capabilities=["CAPABILITY_IAM"],
                ChangeSetName=changeset_id
            )
            cs_id = cs_response['Id']
            changes = self.cloudformation_client.describe_change_set(StackName=cf_stack.stack_name, ChangeSetName=cs_id)

            # delay for 2 seconds while waiting for changeset to create
            time.sleep(2)

            print(colored("The following stack update changes to be applied:", 'cyan'))
            print(colored("================================================================================", 'cyan'))
            print(colored(changes, 'yellow'))
            print(colored("================================================================================", 'cyan'))
            perform_update = query_yes_no("Perform changes?")
            if perform_update:
                self.cloudformation_client.execute_change_set(
                    ChangeSetName=changeset_id,
                    StackName=cf_stack.stack_name
                )

                waiter = CloudformationWaiter(self.cloudformation)
                print(colored('Cloudformation stack update in progress. Please check the AWS console!', color='green'))
                print(colored('Waiting on stack update...', color='magenta'))
                waiter.waiter.wait(StackName=service_name)
                print(colored('Stack update finished!', color='green'))
            else:
                self.cloudformation_client.delete_change_set(StackName=cf_stack.stack_name, ChangeSetName=cs_id)
        except ClientError as e:
            raise NovaError(str(e))
        except WaiterError as e:
            raise NovaError(str(e))

    def create_deployment(self, cd_app, cd_deploy_group, revision_etag, deployment_bucket_name, key):
        response = self.code_deploy_client.create_deployment(
            applicationName=cd_app,
            deploymentGroupName=cd_deploy_group,
            revision={
                'revisionType': 'S3',
                's3Location': {
                    'bucket': deployment_bucket_name,
                    'key': key,
                    'bundleType': 'tgz',
                    'eTag': revision_etag
                }
            }
        )
        print(response)
        return response

    def push_revision(self, deployment_bucket_name, key, code_deploy_app, nova_deploy_dir):
        print(colored("Compressing deployment bundle...", color='cyan'))
        # zip entire nova-deploy directory to {app-name}-{build-id}.tar.gz and push to s3
        tmp_file = tempfile.NamedTemporaryFile(mode="wb", suffix=".gz", delete=False)
        with tarfile.open(tmp_file.name, 'w:gz') as gz_out:
            for f in os.listdir(nova_deploy_dir):
                gz_out.add(os.path.join(nova_deploy_dir, f), arcname=f, progress=tarfile.progressprint)

        print(colored("Pushing deployment bundle '%s' to S3..." % tmp_file.name, color='green'))

        try:
            transfer = S3Transfer(self.s3_client)
            transfer.upload_file(tmp_file.name, deployment_bucket_name, key, callback=ProgressPercentage(tmp_file.name))
        except ClientError as e:
            if 'AccessDenied' in e.message:
                print(colored("Access denied to upload to '%s/%s'" % (deployment_bucket_name, key), 'red'))
                raise

        os.unlink(tmp_file.name)  # clean up the temp file

        print("")  # Need a new-line after transfer progress
        print(colored("Revision '%s' uploaded to S3 for deployment." % key, color='green'))

        revision_etag = self.s3_head(deployment_bucket_name, key).get('ETag')

        self.code_deploy_client.register_application_revision(
            applicationName=code_deploy_app,
            revision={
                'revisionType': 'S3',
                's3Location': {
                    'bucket': deployment_bucket_name,
                    'key': key,
                    'bundleType': 'tgz',
                    'eTag': revision_etag
                }
            }
        )

        return revision_etag

    def kms_generate_data_key(self, kms_key, context):
        try:
            return self.kms_client.generate_data_key(KeyId=kms_key, EncryptionContext=context, NumberOfBytes=64)
        except:
            raise NovaError("Could not generate key using KMS key %s: %s" % (kms_key, str(sys.exc_info()[0])))

    def kms_key_exists(self, kms_key):
        return self.kms_client.describe_key(KeyId=kms_key) is not None

    def kms_decrypt(self, cipher, context):
        try:
            return self.kms_client.decrypt(CiphertextBlob=cipher, EncryptionContext=context)
        except ClientError as e:
            if e.response["Error"]["Code"] == "InvalidCiphertextException":
                if context is None:
                    msg = ("Could not decrypt hmac key with KMS. The credential may "
                           "require that an encryption context be provided to decrypt it.")
                else:
                    msg = ("Could not decrypt hmac key with KMS. The encryption "
                           "context provided may not match the one used when the "
                           "credential was stored.")
            else:
                msg = "Decryption error %s" % e
            raise NovaError(msg)
        except Exception as e:
            raise NovaError("Decryption error %s" % e)

    def create_and_upload_stack_template(self, s3_bucket, service, environment):
        bucket = self.create_bucket(s3_bucket, 'Common Nova service templates bucket does not exist, creating...')

        for s, template in environment.get_stack_templates_used(service, s3_bucket).items():
            if 's3_key' in template:
                template_file = template['filename']
                template_s3_key = template['s3_key']
            else:
                template_version = template['version']
                template_filename = '%s.json' % template['name']
                user_home_dir = os.path.join(os.path.expanduser('~'), '.nova')

                if not os.path.exists(user_home_dir):
                    raise NovaError("Unable to find NOVA templates. Please create templates under '~/.nova'.")

                repo = git.Repo(user_home_dir)
                # If repo is dirty or has un-pushed commits we want to fail here
                if repo.is_dirty() or bool(repo.git.rev_list("origin/master..master")):
                    raise NovaError("You have local modifications to '~/.nova'. These are shared templates! Please create a pull request!")

                template_file = os.path.join(user_home_dir, template_version, template_filename)
                template_s3_key = '%s/%s' % (template_version, template_filename)

            existing_file = None
            if bucket:
                print(colored('Found common Nova service templates bucket. Verifying contents...', color='cyan'))
                checksum = self.__md5(template_file)
                try:
                    existing_file = self.s3_client.head_object(Bucket=s3_bucket, Key=template_s3_key, IfMatch=checksum)
                except ClientError:
                    pass

            if not existing_file:
                print(colored('Common Nova service templates not found. Uploading...', color='cyan'))
                with open(template_file, 'rb') as template_data:
                    self.s3_client.put_object(Bucket=s3_bucket, Key=template_s3_key, Body=template_data)
                print(colored('Contents uploaded!', color='green'))
            else:
                print(colored('Contents verified!', color='green'))

        print(colored('All required templates have been uploaded & verified!', color='green'))

    @staticmethod
    def __md5(fname):
        md5_hash = hashlib.md5()
        with open(fname, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                md5_hash.update(chunk)
        return md5_hash.hexdigest()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import yaml
import atexit
import shutil
import tempfile
from jinja2 import Template
from termcolor import colored
from distutils import dir_util
from os.path import isfile, join

from nova.core import templates, check_latest_version
from nova.core.deploy import LOAD_DOCKER_CONTAINER, START_DOCKER_CONTAINER, VALIDATE
from nova.core.exc import NovaError
from nova.core.spec.custom_scripts import CustomScripts
from nova.core.spec.nova_service_loader import NovaServiceLoader


class DeployStack:

    def __init__(self, environment_name, stack_name, aws_profile, manager_provider,
                 version=None, deploy=True, nova_descriptor_file=None):
        self._environment_name = environment_name
        self._stack_name = stack_name
        self._deploy = deploy
        self._version_manager = manager_provider.version_manager()
        self._docker_manager = manager_provider.docker_manager()
        self._service_manager = NovaServiceLoader(environment_name, nova_descriptor_file)
        self._aws_manager = manager_provider.aws_manager(
            aws_profile or self._service_manager.environment.aws_profile,
            self._service_manager.environment.aws_region
        )
        self._build_id = self._version_manager.current_version(version)
        self._stack = self._service_manager.get_stack(self._stack_name)
        self._code_deploy_app = self._service_manager.code_deploy_app
        if self._stack.deployment_group is None:
            raise NovaError("Environment '%s' stack '%s' does not have 'deployment_group' set!" % (
                self._environment_name, self._stack.name))

        self._nova_deploy_dir = None
        self._custom_scripts = CustomScripts()

    def deploy(self):
        atexit.register(self.__cleanup)
        check_latest_version()

        print("Deploying to account '%s'..." % self._aws_manager.account_alias)
        print(colored("Creating deployment bundle for revision '%s'..." % self._build_id, color='cyan'))

        deployment_bucket_name = self._service_manager.environment.deployment_bucket
        key = "%s/%s.tar.gz" % (self._service_manager.service_name, self._build_id)
        existing_revision = self._aws_manager.s3_head(deployment_bucket_name, key)

        # Create tmp/nova-deploy directory
        self._nova_deploy_dir = tempfile.mkdtemp(prefix="%s-nova-deploy-" % self._service_manager.service_name)

        if existing_revision is None:
            revision_etag = self.__build_upload_revision(deployment_bucket_name, key)
        else:
            print(colored("Existing revision found, deploying...", color='green'))
            revision_etag = existing_revision.get('ETag')

        print(colored("Triggering code-deploy...", color='cyan'))

        if self._deploy:
            self._aws_manager.create_deployment(
                self._code_deploy_app,
                self._stack.deployment_group,
                revision_etag,
                deployment_bucket_name,
                key
            )
            print(colored('CodeDeploy deployment in progress. Please check the AWS console!', color='green'))
        else:
            print(colored('Deployment not triggered, S3 revision uploaded and registered with CodeDeploy.', color='yellow'))

    def __build_upload_revision(self, deployment_bucket_name, key):
        print(colored("No existing revision found, creating...", color='magenta'))
        print(colored("Generating deployment scripts in '%s'..." % self._nova_deploy_dir, color='cyan'))
        self.__create_nova_deploy_dirs()
        self.__create_app_spec()
        # Find docker image and 'save' it to {nova_deploy_dir}/docker/image-{build-id}.tar.gz
        looking_for_tag = "%s:%s" % (self._service_manager.service_name, self._build_id)
        out_file = "%s/docker/%s-%s-docker-image.tar.gz" % (
            self._nova_deploy_dir, self._service_manager.service_name, self._build_id)
        dir_util.mkpath("%s/docker" % self._nova_deploy_dir)
        docker_image = self._docker_manager.save_docker_image_with_tag(looking_for_tag, out_file)
        self.__generate_scripts(docker_image)
        revision_etag = self._aws_manager.push_revision(deployment_bucket_name, key, self._code_deploy_app, self._nova_deploy_dir)
        shutil.rmtree(self._nova_deploy_dir)
        return revision_etag

    def __create_nova_deploy_dirs(self):
        if os.path.exists('nova-scripts'):
            print(colored("Including found custom service deployment scripts", color='green'))
            self._custom_scripts.app_stop_scripts = self.__copy_custom_scripts('nova-scripts/ApplicationStop')
            self._custom_scripts.before_install_scripts = self.__copy_custom_scripts('nova-scripts/BeforeInstall')
            self._custom_scripts.after_install_scripts = self.__copy_custom_scripts('nova-scripts/AfterInstall')
            self._custom_scripts.app_start_scripts = self.__copy_custom_scripts('nova-scripts/ApplicationStart')
            self._custom_scripts.validate_scripts = self.__copy_custom_scripts('nova-scripts/ValidateService')

    def __copy_custom_scripts(self, path_dir):
        if os.path.exists(path_dir):
            dir_util.copy_tree(path_dir, self._nova_deploy_dir)
            return [f for f in os.listdir(path_dir) if isfile(join(path_dir, f))]
        else:
            return []

    def __create_app_spec(self):
        os.mkdir(os.path.join(self._nova_deploy_dir, 'env-vars'))
        files = [
            {
                'source': 'docker/%s-%s-docker-image.tar.gz' % (self._service_manager.service_name, self._build_id),
                'destination': '/tmp'
            }
        ]

        for stack in self._service_manager.environment.stacks:
            os.mkdir(os.path.join(self._nova_deploy_dir, 'env-vars/%s' % stack.stack_type))
            files.extend(self.__render_stack_files(stack))

        app_stop_scripts = [{'location': 'cleanup_space.sh', 'timeout': 300}]
        app_stop_scripts.extend(self._custom_scripts.app_stop_scripts_for_app_spec)

        before_install_scripts = [
            {'location': 'deregister_from_elb.sh'},
            {'location': 'kill_docker_container.sh', 'timeout': 300}
        ]
        before_install_scripts.extend(self._custom_scripts.before_install_scripts_for_app_spec)

        after_install_scripts = [{'location': 'load_docker_container.sh'}]
        after_install_scripts.extend(self._custom_scripts.after_install_scripts_for_app_spec)

        app_start_scripts = [
            {'location': 'start_docker_container.sh', 'timeout': 300},
            {'location': 'register_with_elb.sh'}
        ]
        app_start_scripts.extend(self._custom_scripts.app_start_scripts_for_app_spec)

        validate_scripts = [{'location': 'validate_service.sh'}]
        validate_scripts.extend(self._custom_scripts.validate_scripts_for_app_spec)

        app_spec = {
            'version': 0.0,
            'os': 'linux',
            'files': files,
            'hooks': {
                'ApplicationStop': app_stop_scripts,
                'BeforeInstall': before_install_scripts,
                'AfterInstall': after_install_scripts,
                'ApplicationStart': app_start_scripts,
                'ValidateService': validate_scripts
            }
        }
        with open('%s/appspec.yml' % self._nova_deploy_dir, 'w') as appspecfile:
            yaml.dump(app_spec, appspecfile)

        print(colored("Generated appspec.yml", color='green'))

    def __render_stack_files(self, stack):
        env_vars_script = '#!/usr/bin/env bash\n\n'
        env_vars_script += 'set -e\n\n'

        deployment_options = ''
        if stack.deployment_options is not None:
            for opts in stack.deployment_options:
                for args in opts.items():
                    deployment_options += "{0}={1} ".format(args[0], args[1])
        env_vars_script += 'DOCKER_OPTS="%s"\n' % deployment_options.strip()

        deployment_volumes = ''
        if stack.deployment_volumes is not None:
            for opts in stack.deployment_volumes:
                for args in opts.items():
                    deployment_volumes += "-v {0}:{1} ".format(args[0], args[1])
        env_vars_script += 'DOCKER_VOLS="%s"\n' % deployment_volumes.strip()

        deployment_variables = ''
        if stack.deployment_variables is not None:
            for opts in stack.deployment_variables:
                for args in opts.items():
                    deployment_variables += '-e "{0}={1}" '.format(args[0], args[1])
        env_vars_script += 'DOCKER_VARS="%s"\n' % deployment_variables.strip()

        deployment_arguments = ''
        if stack.deployment_arguments is not None:
            for opts in stack.deployment_arguments:
                for args in opts.items():
                    deployment_arguments += "{0}={1} ".format(args[0], args[1])
        env_vars_script += 'DOCKER_ARGS="%s"\n' % deployment_arguments.strip()

        self.__render_file('env-vars/%s/set-env-vars.sh' % stack.stack_type, env_vars_script)

        return [{
            'source': 'env-vars/%s/set-env-vars.sh' % stack.stack_type,
            'destination': '/opt/nova/environments/%s' % stack.stack_type
        }]

    def __generate_scripts(self, docker_image):
        deploy_scripts_path = os.path.join(templates.__path__[0], 'deploy-scripts')
        for f in os.listdir(deploy_scripts_path):
            file_name = os.path.join(deploy_scripts_path, f)
            shutil.copy(file_name, self._nova_deploy_dir)

        print(colored("Including default common scripts", color='cyan'))

        load_data = {
            'image': docker_image.get("RepoTags")[0],
            'image_filename': '%s-%s-docker-image.tar.gz' % (self._service_manager.service_name, self._build_id)
        }
        self.__render_script('load_docker_container.sh', LOAD_DOCKER_CONTAINER, load_data)
        print(colored("Load docker image script written.", color='green'))

        start_data = {
            'service_name': self._service_manager.service_name,
            'port': self._service_manager.service_port,
            'stack_type': self._stack.stack_type,
            'stack_name': self._stack.name,
            'environment_name': self._service_manager.environment.name,
            'image': docker_image.get("RepoTags")[0]
        }
        self.__render_script('start_docker_container.sh', START_DOCKER_CONTAINER, start_data)
        print(colored("Start docker container script written.", color='green'))

        validate_data = {
            'port': self._service_manager.service_port,
            'healthcheck_url': self._service_manager.service_healthcheck_url
        }
        self.__render_script('validate_service.sh', VALIDATE, validate_data)
        print(colored("Validate service script written.", color='green'))

    def __render_script(self, script_filename, template, data):
        script_template = Template(template, trim_blocks=True)
        script_body = str(script_template.render(data)).strip()
        self.__render_file(script_filename, script_body)

    def __render_file(self, filename, file_body):
        with open(os.path.join(self._nova_deploy_dir, filename), 'w') as f:
            f.write(file_body)
            f.write(os.linesep)

    def __cleanup(self):
        try:
            if os.path.exists(self._nova_deploy_dir):
                print(colored("Cleaning up deployment bundle...", color='green'))
                shutil.rmtree(self._nova_deploy_dir)
        except Exception as e:
            raise NovaError(e)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import subprocess
from termcolor import colored
from docker import Client as DockerClient
from docker.utils import kwargs_from_env

from nova.core import NovaError


class DockerManager(object):

    def __init__(self):
        self._docker_client = None

    @property
    def docker_client(self):
        if self._docker_client is None:
            kwargs = kwargs_from_env()
            try:
                kwargs['tls'].assert_hostname = False
            except KeyError:
                pass
            kwargs['version'] = 'auto'
            self._docker_client = DockerClient(**kwargs)
        return self._docker_client

    def images(self):
        return self.docker_client.images()

    def find_image(self, looking_for_tag):
        matching_images = [d for d in self.docker_client.images() if d['RepoTags'] is not None and looking_for_tag in d['RepoTags']]
        if not len(matching_images) == 1:
            raise NovaError("Could not find a docker image with a tag for: '%s'" % looking_for_tag)
        else:
            return matching_images[0]

    def save_docker_image_with_tag(self, looking_for_tag, out_file):
        docker_image = self.find_image(looking_for_tag)
        print(colored("Found docker image '%s'." % looking_for_tag, color='green'))
        print(colored("Exporting docker image to include in bundle...", color='cyan'))
        save_cmd = ['docker', 'save', '-o', out_file, looking_for_tag]
        p = subprocess.Popen(save_cmd, stdout=subprocess.PIPE)
        while p.poll() is None:
            print(colored(p.stdout.readline(), color='magenta'))
        print(colored(p.stdout.read(), color='magenta'))
        return docker_image



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import os
import yaml

from cerberus import Validator

from nova.core import spec
from nova.core.exc import NovaError
from nova.core.spec import yaml_include
from nova.core.spec.service import Service


class NovaServiceLoader:

    def __init__(self, environment_name, nova_descriptor_file=None):
        self._nova_descriptor_file = nova_descriptor_file or 'nova.yml'
        self._environment_name = environment_name
        self._environment = None
        self._codedeploy_app = None

        self.templates_used = dict()
        yaml.add_constructor("!include", yaml_include)

        with open(os.path.join(spec.__path__[0], 'nova_service_schema.yml'), 'r') as schemaYaml:
            schema = yaml.load(schemaYaml)

        v = Validator(schema)
        try:
            with open(self._nova_descriptor_file, 'r') as novaYaml:
                self.service_spec = yaml.safe_load(novaYaml)

            # Validate loaded dictionary
            valid = v.validate(self.service_spec)
            if not valid:
                raise NovaError("Invalid nova service descriptor file '%s': %s" % (self._nova_descriptor_file, v.errors))
            else:
                self.service = Service.load(self.service_spec)
                self.service_name = self.service.name
                self.service_port = self.service.port
                self.service_healthcheck_url = self.service.healthcheck_url
        except IOError:
            raise NovaError("No nova service descriptor found at '%s'" % self._nova_descriptor_file)

    @property
    def environment(self):
        if self._environment is None:
            self._environment = self.service.get_environment(self._environment_name)
        return self._environment

    @property
    def code_deploy_app(self):
        if self.environment.deployment_application_id is None:
            raise NovaError("Environment '%s' does not have 'deployment_application_id' set!" % self._environment_name)

        if self._codedeploy_app is None:
            self._codedeploy_app = self.environment.deployment_application_id
        return self._codedeploy_app

    def get_stack(self, stack_name):
        return self.environment.get_stack(stack_name)

    def set_code_deploy_app(self, environment_name, code_deploy_app_id):
        self.service.set_code_deploy_app(environment_name, code_deploy_app_id)



from setuptools.command.test import test as TestCommand
from setuptools import setup, find_packages
from pip.req import parse_requirements

import os
import sys
import glob

conf = []

for name in glob.glob('config/plugins.d/*.conf'):
    conf.insert(1, name)

userdir_conf = os.path.expanduser("~/.nova")
if not os.path.exists(userdir_conf):
    os.makedirs(userdir_conf)

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt', session=False)


class PyTest(TestCommand):

    user_options = [('cov=', None, 'Run coverage'), ('cov-xml=', None, 'Generate junit xml report'), ('cov-html=',
                    None, 'Generate junit html report'), ('junitxml=', None, 'Generate xml of test results')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.cov = None
        self.cov_xml = False
        self.cov_html = False
        self.junitxml = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        if self.cov is not None:
            self.cov = ['--cov', self.cov, '--cov-report', 'term-missing']
            if self.cov_xml:
                self.cov.extend(['--cov-report', 'xml'])
            if self.cov_html:
                self.cov.extend(['--cov-report', 'html'])
        if self.junitxml is not None:
            self.junitxml = ['--junitxml', self.junitxml]

    def run_tests(self):
        try:
            import pytest
        except:
            raise RuntimeError('py.test is not installed, run: pip install pytest')
        params = {'args': self.test_args}
        if self.cov:
            params['args'] += self.cov
        if self.junitxml:
            params['args'] += self.junitxml
        params['args'] += ['--doctest-modules', 'nova', '-s', '-vv']
        errno = pytest.main(**params)
        sys.exit(errno)


setup(name='gilt-nova',
      version='11.1.0',
      description="Collection of utilities to easily deploy services to AWS.",
      long_description="Collection of utilities to easily deploy services to AWS.",
      license='MIT',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',

          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Topic :: Software Development :: Build Tools',

          'License :: OSI Approved :: MIT License',

          'Operating System :: POSIX :: Linux',

          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',

          'Programming Language :: Python :: Implementation :: CPython',
      ],
      keywords='AWS CodeDeploy Cloudformation Deployment Gilt',
      author='Gilt Groupe',
      author_email='gilt-nova-dev@googlegroups.com',
      url='https://github.com/gilt/nova',
      test_suite='tests',
      packages=find_packages(exclude=['ez_setup', 'contrib', 'docs', 'tests', 'tests.*']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[str(ir.req) for ir in install_reqs],
      cmdclass={'test': PyTest},
      tests_require=['pytest-cov', 'pytest', 'nose', 'mock'],
      command_options={
          'test': {
              'test_suite': ('setup.py', 'tests'), 'cov': ('setup.py', 'nova'),
              'cov_xml': ('setup.py', True),
              'junitxml': ('setup.py', 'junit.xml')
          }
      },
      data_files=[(userdir_conf, ['config/nova.conf']), (os.path.expanduser('~/.nova/plugins.d'), conf)],
      setup_requires=['flake8'],
      entry_points="""
          [console_scripts]
          nova = nova.cli.main:main
      """,
      namespace_packages=[])



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from termcolor import colored

from nova.core import check_latest_version
from nova.core.exc import NovaError
from nova.core.managers.version_manager import VersionManager
from nova.core.spec.nova_service_loader import NovaServiceLoader


def green(message):
    return colored(message, color='green')


class UpdateStack:

    def __init__(self, aws_profile, environment_name, manager_provider,
                 cf_template_out=None, nova_descriptor_file=None, include_docker=True):
        check_latest_version()

        print(colored("Creating cloudformation scripts...", color='cyan'))
        self.environment_name = environment_name
        self._service_manager = NovaServiceLoader(environment_name, nova_descriptor_file)
        self._aws_manager = manager_provider.aws_manager(
            aws_profile or self._service_manager.environment.aws_profile,
            self._service_manager.environment.aws_region
        )

        self._s3_bucket = 'nova-deployment-templates-%s' % self._aws_manager.account_alias

        self.cloudformation_template = self._service_manager.service.to_cfn_template(
            self._service_manager.environment,
            self._s3_bucket,
            self._aws_manager,
            cf_template_out,
            include_docker
        )

    def update(self):
        self._aws_manager.create_and_upload_stack_template(self._s3_bucket, self._service_manager.service,
                                                           self._service_manager.environment)
        changeset_id = "cs%s" % VersionManager().current_version().replace(".", "x")
        cf_stack = self._aws_manager.get_stack(self._service_manager.service_name)
        performed_update = self._aws_manager.update_stack(
            self._service_manager.service_name,
            self.cloudformation_template,
            changeset_id,
            cf_stack
        )

        if performed_update:
            stack = self._aws_manager.get_stack(self._service_manager.service_name)
            if stack.stack_status == "UPDATE_COMPLETE":
                if stack.outputs is not None:
                    for output in stack.outputs:
                        if output.get('OutputKey') == 'CodeDeployApp':
                            code_deploy_app_id = output.get('OutputValue')
                            print("Found code-deploy app: %s" % code_deploy_app_id)
                            self._service_manager.service.set_code_deploy_app(self.environment_name, code_deploy_app_id)
                            print(green("Please update nova.yml environment with 'deployment_application_id' manually."))
                            break

                        for stack in self._service_manager.environment.stacks:
                            if output.get('OutputKey') == stack.name.title() + 'DeploymentGroup':
                                code_deploy_app_id = output.get('OutputValue')
                                print(colored("Found code-deploy deployment group for stack '%s': %s" % (
                                    stack.name, code_deploy_app_id), color='green'))
                                self._service_manager.service.set_code_deploy_app(self.environment_name, code_deploy_app_id)
                                print(green("Please update nova.yml environment with 'deployment_application_id' manually."))
                            else:
                                print(colored(
                                    "Unable to find code-deploy deployment group for stack '%s' in the stack output" % stack.name,
                                    'yellow'))
                                print(colored("Please update nova.yml stack with the 'deployment_group' manually.",
                                              color='yellow'))

                    else:
                        print(colored('Unable to find code-deploy application in the stack output', 'yellow'))
                        print(colored("Please update nova.yml environment with 'deployment_application_id' manually.",
                                      color='yellow'))
                else:
                    print(colored("Please update nova.yml environment with 'deployment_application_id' manually.",
                                  color='yellow'))
                    print(colored("and your stacks with their respective 'deployment_group' manually.", color='yellow'))

            else:
                raise NovaError("Stack update was un-successful: %s - %s" % (stack.stack_status, stack.stack_status_reason))
